import 'dart:async';

import 'package:fis_i18n/i18n.dart';
import 'package:fis_measure/interfaces/enums/annotation.dart';
import 'package:fis_measure/interfaces/enums/operate.dart';
import 'package:fis_measure/interfaces/process/player/play_controller.dart';
import 'package:fis_measure/interfaces/process/standard_line/calibration.dart';
import 'package:fis_measure/interfaces/process/workspace/application.dart';
import 'package:fis_measure/interfaces/process/workspace/exam_info.dart';
import 'package:fis_measure/process/workspace/measure_controller.dart';
import 'package:fis_measure/process/workspace/measure_data_controller.dart';
import 'package:fis_measure/process/workspace/measure_handler.dart';
import 'package:fis_measure/process/workspace/third_part/application.dart';
import 'package:fis_measure/view/measure/measure_view_controller.dart';
import 'package:fis_measure/view/menu_button_group/menu_button_group.dart';
import 'package:fis_measure/view/player/controller.dart';
import 'package:fis_ui/base_define/page.dart';
import 'package:fis_ui/index.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:get/get_connect/http/src/utils/utils.dart';
import 'package:vid/us/vid_us_image.dart';

/// 测量工具 注释工具 切换按钮
class OperateTypeChangeButton extends StatefulWidget implements FWidget {
  const OperateTypeChangeButton({
    super.key,
    required this.businessParent,
  });

  final FPage businessParent;

  @override
  State<OperateTypeChangeButton> createState() =>
      OperateTypeChangeButtonState();
}

// 两个按钮共有三种状态
enum OperateButtonState { isMeasure, isAnnotation, allClose }

class OperateTypeChangeButtonState extends State<OperateTypeChangeButton>
    with TickerProviderStateMixin {
  late final measureHandler = Get.find<MeasureHandler>();
  // late TabController tabController;
  late final measureData = Get.find<MeasureDataController>();
  bool isShowMeasureButton = true;
  OperateButtonState operateButtonState = OperateButtonState.isMeasure;

  /// 全部图片list
  final measureController = Get.find<MeasureController>();
  VidPlayerController playerController =
      Get.find<IPlayerController>() as VidPlayerController;

  @override
  void initState() {
    super.initState();
    measureHandler.annotationsLoaded.addListener(onAnnotationsLoaded);
    measureController.imageLoaded.addListener(_onImageLoaded);
  }

  @override
  void dispose() {
    super.dispose();
    measureHandler.annotationsLoaded.removeListener(onAnnotationsLoaded);
    measureController.imageLoaded.removeListener(_onImageLoaded);
    // tabController.dispose();
  }

  void initTab() {
    final application = Get.find<IApplication>();

    ///TODO:[Gavin] 虽然屏蔽了多幅，但是切换至多幅时候仍会存在问题【虽然只显示注释，但是默认会被切换至第一个测量项】
    ///原因是Tab页切换优化为了FOffstage的方式，Measure Tab 实际没有被 dispose
    ///比较妥协的临时解决方案是，触发切换注释时加上 500ms 延时【line 178:180】
    if (application.isSingleFrame && application.visuals.length > 1) {
      //多幅vid
      changeTypeTo(MeasureOperateType.annotation);
      setState(() {
        isShowMeasureButton = false;
        operateButtonState = OperateButtonState.isAnnotation;
      });
    } else {
      _enterMeasure(() {
        measureHandler.toolPanelState = false;
        measureHandler.canMeasureDrawing = false;
        setState(() {
          operateButtonState = OperateButtonState.allClose;
        });
      });
    }
  }

  void changeTypeTo(e) {
    measureHandler.currOperateType = e;
    if (e == MeasureOperateType.measure) {
      final measureMetaController = Get.find<MeasureMetaController>();
      measureMetaController.setAvailableModes(measureData.currentMode);
      // 用于切换到第一个测量项
      measureMetaController.updateItemList();
    } else {
      final application = Get.find<IApplication>();
      Future.delayed(const Duration(milliseconds: 500), () {
        application.switchAnnotation(AnnotationType.label, "test");
      });
    }
  }

  // 应该在图像注释获取完成后再触发initTab
  void onAnnotationsLoaded(Object sender, _) {
    if (!mounted) return;
    initTab();
  }

  void _onFrameLoadedAfterFirst(Object sender, VidUsImage? e) {
    if (!mounted) return;
    initTab();
    playerController.firstFrameLoaded.removeListener(_onFrameLoadedAfterFirst);
  }

  void _onImageLoaded(Object sender, ExamImageInfo? e) {
    playerController = Get.find<IPlayerController>() as VidPlayerController;
    playerController.firstFrameLoaded.addListener(_onFrameLoadedAfterFirst);
  }

  @override
  void didUpdateWidget(covariant OperateTypeChangeButton oldWidget) {
    super.didUpdateWidget(oldWidget);
  }

  onTapMeasure() {
    if (operateButtonState == OperateButtonState.isMeasure) {
      measureHandler.toolPanelState = false;
      setState(() {
        operateButtonState = OperateButtonState.allClose;
      });
    } else {
      final rollbackFn = _generateStatesRollbackFn();
      _enterMeasure(rollbackFn);
    }
  }

  onTapAnnotation() {
    if (operateButtonState == OperateButtonState.isAnnotation) {
      measureHandler.toolPanelState = false;
      setState(() {
        operateButtonState = OperateButtonState.allClose;
      });
    } else {
      measureHandler.toolPanelState = true;
      changeTypeTo(MeasureOperateType.annotation);
      setState(() {
        operateButtonState = OperateButtonState.isAnnotation;
      });
    }
  }

  /// 进入测量
  Future<void> _enterMeasure(VoidCallback rejectCallback) async {
    _setEnterMeasureState(false); // 预进入

    final checked = await _checkStandardLineNotEmpty();
    if (checked) {
      _setEnterMeasureState();
    } else {
      rejectCallback();
    }
  }

  /// 检查第三方图像校准线信息是否空
  Future<bool> _checkStandardLineNotEmpty() {
    final application = Get.find<IApplication>();
    if (application.isThirdPart) {
      final standardLine = (application as ThirdPartApplication).standardLine;
      if (standardLine.currentPixelSpacing.isEmpty) {
        final completer = Completer<bool>();
        FConfirmAlert.show(
          context: context,
          title: i18nBook.common.tip.t,
          subTitle: i18nBook.measure.thirdImageNeedCalibratingTips.t,
          businessParent: widget.businessParent,
          onConfirm: () {
            final calibrationController =
                Get.find<IStandardLineCalibrationController>();

            void onEditStateChanged(
                Object _, StandardLineCalibrationEditState e) {
              void finish(bool result) {
                calibrationController.editStateChanged
                    .removeListener(onEditStateChanged);
                completer.complete(result);
              }

              if (e == StandardLineCalibrationEditState.done) {
                finish(true);
              } else if (e == StandardLineCalibrationEditState.inactive) {
                finish(false);
              } else {
                // no process
              }
            }

            // 开始监听
            calibrationController.editStateChanged
                .addListener(onEditStateChanged);

            // 进入编辑
            calibrationController.enterEditMode();
          },
          onCancel: () {
            completer.complete(false);
          },
        );
        return completer.future;
      }
    }
    return Future.value(true);
  }

  /// 生成状态回滚函数
  VoidCallback _generateStatesRollbackFn() {
    final toolPanelStateSnapshot = measureHandler.toolPanelState;
    final canMeasureDrawingSnapshot = measureHandler.canMeasureDrawing;
    final operateButtonStateSnapshot = operateButtonState;
    final operateTypeSnapshot = measureHandler.currOperateType;
    void fn() {
      measureHandler.toolPanelState = toolPanelStateSnapshot;
      measureHandler.canMeasureDrawing = canMeasureDrawingSnapshot;
      changeTypeTo(operateTypeSnapshot);
      setState(() {
        operateButtonState = operateButtonStateSnapshot;
      });
    }

    return fn;
  }

// 进入测量状态
  void _setEnterMeasureState([bool showToolPanel = true]) {
    if (measureHandler.toolPanelState != showToolPanel) {
      measureHandler.toolPanelState = showToolPanel;
    }
    if (measureHandler.canMeasureDrawing == false) {
      measureHandler.canMeasureDrawing = true;
    }
    if (measureHandler.currOperateType != MeasureOperateType.measure) {
      changeTypeTo(MeasureOperateType.measure);
    }
    if (operateButtonState != OperateButtonState.isMeasure) {
      setState(() {
        isShowMeasureButton = true;
        operateButtonState = OperateButtonState.isMeasure;
      });
    }
  }

  bool get isCurrMeasure => operateButtonState == OperateButtonState.isMeasure;
  bool get isCurrAnnotation =>
      operateButtonState == OperateButtonState.isAnnotation;

  @override
  FWidget build(BuildContext context) {
    return FSizedBox(
        width: 60,
        child: FColumn(children: [
          if (isShowMeasureButton)
            FCustomLeftTooltip(
              textStyle: const TextStyle(fontSize: 16, color: Colors.white),
              height: 36,
              message: i18nBook.measure.measureTool.t,
              verticalOffset: 0,
              margin: const EdgeInsets.only(right: 10),
              decoration: BoxDecoration(
                color: Colors.black.withOpacity(1.0),
                borderRadius: BorderRadius.circular(4),
              ),
              child: FInkWell(
                onTap: onTapMeasure,
                child: FContainer(
                  height: 34,
                  margin: const EdgeInsets.only(
                    top: 5,
                  ),
                  padding:
                      const EdgeInsets.symmetric(horizontal: 10, vertical: 5),
                  decoration: BoxDecoration(
                    color: isCurrMeasure ? Colors.grey : Colors.transparent,
                    borderRadius: BorderRadius.circular(5),
                  ),
                  child: FIcon(
                    Icons.straighten,
                    color: isCurrMeasure ? Colors.white : Colors.white,
                  ),
                ),
              ),
            ),
          FCustomLeftTooltip(
            textStyle: const TextStyle(fontSize: 16, color: Colors.white),
            height: 36,
            message: i18nBook.measure.annotationTool.t,
            verticalOffset: 0,
            margin: const EdgeInsets.only(right: 10),
            decoration: BoxDecoration(
              color: Colors.black.withOpacity(1.0),
              borderRadius: BorderRadius.circular(4),
            ),
            child: FInkWell(
              onTap: onTapAnnotation,
              child: FContainer(
                height: 34,
                margin: const EdgeInsets.only(
                  top: 5,
                ),
                padding:
                    const EdgeInsets.symmetric(horizontal: 10, vertical: 5),
                decoration: BoxDecoration(
                  color: isCurrAnnotation ? Colors.grey : Colors.transparent,
                  borderRadius: BorderRadius.circular(5),
                ),
                child: FIcon(
                  Icons.font_download,
                  color: isCurrAnnotation ? Colors.white : Colors.white,
                ),
              ),
            ),
          )
        ]));
  }
}
